<?php

namespace Drupal\packagist\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'packagist.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'packagist_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('packagist.settings');
    $form['direct_execution'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Direct execution'),
      // @todo: add a warning into description
      '#description' => $this->t('Execute server-wrapper.sh script directly (not recommended).'),
      '#default_value' => $config->get('direct_execution'),
    ];
    // allow to edit host and port when direct_execution disabled only
    $form['host'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Host'),
      // @todo: check if domain names are allowed (also check for http/s not included)
      '#description' => $this->t('Backend host name or IP address.'),
      '#default_value' => $config->get('host'),
      '#states' => [
        'disabled' => [
          ':input[name="direct_execution"]' => ['checked' => TRUE],
        ],
      ],
      '#required' => TRUE,
    ];
    $form['port'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Port'),
      '#description' => $this->t('Backend port number.'),
      '#default_value' => $config->get('port'),
      '#states' => [
        'disabled' => [
          ':input[name="direct_execution"]' => ['checked' => TRUE],
        ],
      ],
      '#required' => TRUE,
    ];
    $form['enable_ssl'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable SSL encryption'),
      '#description' => $this->t('Enable ssl encryiption for communication between client and server hosts (if enabled, must be supported on backend side).'),
      '#default_value' => $config->get('enable_ssl'),
      '#states' => [
        'disabled' => [
          ':input[name="direct_execution"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['auth_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Authentication key'),
      '#description' => $this->t('Authentication key for the backend.'),
      '#default_value' => $config->get('auth_key'),
      '#states' => [
        'disabled' => [
          ':input[name="direct_execution"]' => ['checked' => TRUE],
        ],
      ],
      '#required' => TRUE,
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config('packagist.settings')
      ->set('direct_execution', $form_state->getValue('direct_execution'))
      ->set('host', $form_state->getValue('host'))
      ->set('port', $form_state->getValue('port'))
      ->set('enable_ssl', $form_state->getValue('enable_ssl'))
      ->set('auth_key', $form_state->getValue('auth_key'))
      ->save();
  }

}
